# -*- coding:Utf-8 -*-
# ./addons/eventscripts/_libs/python/hudlib/hinttext.py

import es, psyco, langlib, gamethread
psyco.full()

from lines import Lines
from players import players
path = es.getString("eventscripts_gamedir")



class HintText(Lines):
    """
        * HintText :
            - update : update box
            - kill : delete box
            - hide : hide box
            - send : send box
            - view : allow to know if a player is watching this box

            - addline : add a line
            - insline : insert a line
            - modline : modifie a line
            - delline : remove a line

            . type : get box type
            . timeout : set display time
            . callback : set callback funtion
            . callback : set callback funtion
    """

    def __init__(self):
        # initialize Lines object
        Lines.__init__(self)
        # internal attributes
        self.boxblock = ""
        self.userblock = ""
        self.shellname = ""
        self.firstsend = True
        self.quicksend = False
        self._language = False
        self.timeoutfire = False
        self.timeouthread = "_hl-%s_" % id(self)

        # writable attributes
        self.defaultlang = None
        self.usertimeout = None
        self.boxtimeout = None
        self.timeout = 0

    def __getattr__(self, name):
        # get box type
        if name == "type": return "hinttext"

    def _timeout(self, users):
        self.timeoutfire = True
        # execute "boxtimeout" function
        if callable(self.boxtimeout): self.boxtimeout(self)
        self.timeoutfire = False
        # for all players viewing HintText box
        for userid in users:
            # execute hideMsgBox function with parameter "timeout"
            players.regPlayer(userid).hideMsgBox(self, "timeout")

    def update(self):
        # for all players
        for player in players.values():
            # if HintText is registered
            if player.hinttext.first() == self:
                # if player was viewing it
                if player.hinttextview:
                    # refresh it
                    player.hintstring = ""
                    player.sendMsgBox(self)

    def kill(self):
        # kill timeout thread
        gamethread.cancelDelayed(self.timeouthread)
        # hide HintText for all registered players
        for player in players.values():
            player.hideMsgBox(self, "delete")

    def hide(self, users):
        # hide HintText for all users if it registered
        for userid in getUsers(users):
            players.regPlayer(userid).hideMsgBox(self, "hide")

    def send(self, users, text = None):
        # create or update text
        if text: self.string = str(text)
        # set firstsend status
        elif self.firstsend: self.firstsend = False

        users = getUsers(users)
        # if timeout > 0
        if self.timeout > 0:
            gamethread.cancelDelayed(self.timeouthread)
            # delaye hideMsgBox function
            gamethread.delayedname(self.timeout, self.timeouthread, self._timeout, users)

        # register HintText for all users
        for userid in users:
            # get player
            player = players.regPlayer(userid)
            player.hintstring = ""
            # store HintText object
            player.hinttext.add(self)
            # send HintText
            player.sendMsgBox(self)

    def view(self, userid):
        userid = int(userid)
        # if player is registered
        if players.has_key(userid):
            player = players[userid]
            # if player is viewing a HintText box
            if player.hinttextview:
                # return if player is currently viewing this box
                return player.hinttext.first() == self
        return False

    def viewers(self, isviewed = True):
        users = set()
        # if userid list must contains only players that viewing HinText box
        if isviewed:
            # for each registered player
            for userid, player in players.items():
                # ad the if player is currently viewing the HinText box
                if self == player.hinttext.first(): users.add(userid)

        # else
        else:
            # for each registered player
            for userid, player in players.items():
                # add the userid if the HinText box is viewed or queued by the player
                if self in player.hinttext: users.add(userid)
        # return userid list
        return list(users)

    def language(self, directory = ""):
        # if directory is false
        if not directory: self._language = False
        else:
            # format directory
            if directory.startswith("../"):
                directory = path + directory.replace("../", "/")
            # load language file
            self._language = langlib.Strings(directory)
            if not self._language:
                self._language = False
                raise ValueError, 'hudlib cannot use language file "%s"' % directory

# get users (iterable)
getUsers = lambda users: [users] if str(users).isdigit() else users



# return HintText object
def box():
    return HintText()

# quick send
def send(users, text, time = 6):
    hudbox = HintText()
    hudbox.timeout = time
    hudbox.quicksend = True
    hudbox.send(users, str(text))

# return True/False if a player watching a HintText box
def view(userid):
    userid = int(userid)
    if not players.has_key(userid): return False
    return players[userid].hinttextview