# -*- coding:Utf-8 -*-
# ./addons/eventscripts/_libs/python/hudlib/player.py

import es, psyco, usermsg, langlib, gamethread
from hudqueue import HudQueue
psyco.full()

defaultlang = langlib.getDefaultLang()



class Player(object):
    """
        Player object
            - store player's info (userid, threads)
            - store Key/HintText objects (HudQueue)
            - send and hide Key/HintText boxes
    """

    def __init__(self, userid):
        # store userid and user language
        self.userid = userid
        self.userlang = es.getclientvar(userid, "cl_language")
        if not self.userlang: self.userlang = defaultlang
        else: self.userlang = langlib.getLangAbbreviation(self.userlang)

        # store current box string
        self.hintstring = ""
        self.keyhintstring = ""
        # store names of threads
        self.hinttextthread = "_hl-ht_" + str(userid)
        self.keyhinttextthread = "_hl-kht_" + str(userid)

        # create HudQueue lists and "view" statuses
        self.hinttextview = False
        self.hinttext = HudQueue()
        self.keyhinttextview = False
        self.keyhinttext = HudQueue()

    def disconnect(self):
        # kill user's threads
        gamethread.cancelDelayed(self.hinttextthread)
        gamethread.cancelDelayed(self.keyhinttextthread)
        # call functions with "disconnect" as value for reason
        for hudbox in self.hinttext + self.keyhinttext:
            if callable(hudbox.usertimeout):
                hudbox.usertimeout(self.userid, "disconnect", hudbox)


    def clearZones(self):
        """ this will clear HintText and KeyHintText zones !! """
        # create ResetHUD
        es.usermsg("create", "_hl-clear_", "ResetHUD")
        es.usermsg("write", "byte", "_hl-clear_", -1)
        # reset zones
        es.usermsg("send", "_hl-clear_", self.userid)
        # delete ResetHUD
        es.usermsg("delete", "_hl-clear_")

    def createString(self, hudbox):
        # if box is not multilangual
        if not hudbox._language:
            # self._string = hudbox.string
            if hudbox.string: return hudbox.string
            return "\n".join(hudbox)

        # else
        else:
            tokens = []
            lang = defaultlang if hudbox.defaultlang is None else hudbox.defaultlang
            for section, opts in hudbox:
                # try to get text in player's lang
                if hudbox._language[section].has_key(self.userlang):
                    tokens.append(hudbox._language(section, opts, self.userlang))
                elif hudbox._language[section].has_key(lang):
                    tokens.append(hudbox._language(section, opts, lang))
                # else, raise operation
                else: raise KeyError, 'hudlib: cannot use default language "%s"' % lang
            return "\n".join(tokens)


    def string(self, hudbox):
        if hudbox.type == "hinttext":
            # if string is not updated
            if not self.hintstring:
                # update and return HintText text
                self.hintstring = self.createString(hudbox)
            return self.hintstring

        elif hudbox.type == "keyhinttext":
            # if string is not updated
            if not self.keyhintstring:
                # update and return KeyHintText text
                self.keyhintstring = self.createString(hudbox)
            return self.keyhintstring
        return ""

    def sendHudMsg(self, hudbox):
        # if text exists
        string = self.string(hudbox)
        if string:
            if hudbox.type == "hinttext":
                usermsg.hudhint(self.userid, string)
            elif hudbox.type == "keyhinttext":
                usermsg.keyhint(self.userid, string)


    def sendMsgBox(self, hudbox):
        if hudbox is not None:
            if hudbox.type == "hinttext":
                if self.hinttext.first() == hudbox:
                    self.hinttextview = True
                    # try to send box
                    self.sendHudMsg(hudbox)
                    # repeat operation every 4 seconds
                    gamethread.cancelDelayed(self.hinttextthread)
                    gamethread.delayedname(4, self.hinttextthread, self.sendMsgBox, hudbox)

            elif hudbox.type == "keyhinttext":
                if self.keyhinttext.first() == hudbox:
                    self.keyhinttextview = True
                    # try to send box
                    self.sendHudMsg(hudbox)
                    # repeat operation every 4 seconds
                    gamethread.cancelDelayed(self.keyhinttextthread)
                    gamethread.delayedname(4, self.keyhinttextthread, self.sendMsgBox, hudbox)

    def hideMsgBox(self, hudbox, reason = None):
        if hudbox in self.hinttext:
            if not self.hinttext.index(hudbox):
                # kill HintText thread
                gamethread.cancelDelayed(self.hinttextthread)
            # remove HintText from list
            self.hinttext.remove(hudbox)

            # if function was called for a reason
            if reason:
                # if a function must be call
                if callable(hudbox.usertimeout):
                    hudbox.usertimeout(self.userid, reason, hudbox)

            # update zone
            if not hudbox.shellname: self.updateZones(hudbox)
            else: gamethread.queue(self.updateZones, hudbox)

        elif hudbox in self.keyhinttext:
            if not self.keyhinttext.index(hudbox):
                # kill KeyHintText thread
                gamethread.cancelDelayed(self.keyhinttextthread)
            # remove KeyHintText from list
            self.keyhinttext.remove(hudbox)

            # if function was called for a reason
            if reason:
                # if a function must be call
                if callable(hudbox.usertimeout):
                    hudbox.usertimeout(self.userid, reason, hudbox)

            # update zone
            if not hudbox.shellname: self.updateZones(hudbox)
            else: gamethread.queue(self.updateZones, hudbox)

    def updateZones(self, oldbox):
        if oldbox.type == "hinttext":
            if not self.hinttext: 
                self.hinttextview = False
                # clear HintText zone
                self.clearZones()
                # refresh KeyHintText zone
                self.sendMsgBox(self.keyhinttext.first())
            # if another HintText box was queued, send it
            else:
                hudbox = self.hinttext.first()
                if oldbox <> hudbox: self.sendMsgBox(oldbox)

        elif oldbox.type == "keyhinttext":
            if not self.keyhinttext: 
                self.keyhinttextview = False
                # clear KeyHintText zone
                self.clearZones()
                # refresh HintText zone
                self.sendMsgBox(self.hinttext.first())
            # if another KeyHintText box was queued, send it
            else:
                hudbox = self.hinttext.first()
                if oldbox <> hudbox: self.sendMsgBox(oldbox)
