# coding:Utf-8
import es, psyco, hudlib, keyvalues, playerlib
psyco.full()



# script's dict
hintTextDict = {}
keyHintTextDict = {}

# script's cvars
cvUserid = es.ServerVar("_hud_userid", 0, "HudLib: return userid")
cvReason = es.ServerVar("_hud_reason", "", "HudLib: return reason")
cvHudname = es.ServerVar("_hud_hudname", "", "HudLib: return name of hud")
cvHudtype = es.ServerVar("_hud_hudtype", "", "HudLib: return type of hud")

# lambda functions
digitArg = lambda arg: int(arg) if arg.isdigit() else False
getUsers = lambda users: playerlib.getUseridList(users) if users.startswith("#") else users

# sub commands syntax
subcommands = (
    'quicksend <users> <"text"> [time]',
    "exists <variable> <hudname>",
    "create <hudname>",
    "delete <hudname>",

    "language <hudname> <path>",
    'defaultlang <hudname> <0/"lang">',

    "send <hudname> <users>",
    "hide <hudname> <users>",
    "update <hudname>",
    "view [hudname] <variable> <userid>",
    "viewers <hudname> <keygroup> [isviewed]",
    "timeout <hudname> <time>",
    "boxtimeout <hudname> <script>/<block>",
    "usertimeout <hudname> <script>/<block>",

    'addline <hudname> <"text"> [keygroup [update]]',
    'insline <hudname> <index> <"text"> [keygroup [update]]',
    'modline <hudname> <index> <"text"> [keygroup [update]]',
    "delline <hudname> <index> [update]",
    'getindex <hudname> <variable> <"text">',
    "getstring <hudname> <variable> <index>",
    "getcount <hudname> <variable>")

def load():
    # register "hinttext" command
    if not es.exists("command", "hinttext"):
        es.regcmd("hinttext", "hud/hintTextCmd", "HudLib: command for handling HintText zone")
    # register "keyhinttext" command
    if not es.exists("command", "keyhinttext"):
        es.regcmd("keyhinttext", "hud/keyHintTextCmd", "HudLib: command for handling KeyHintText zone")



def hintTextCmd():
    count = es.getargc()
    if count < 3:
        es.dbgmsg(0, "HudLib: hinttext commands :\n\n  - " + "\n  - ".join(subcommands) + "\n")
    else:
        # get command and first parameter
        command = es.getargv(1)
        parameter = es.getargv(2)

        # for quick send
        if command == "quicksend":
            time = es.getargv(4)
            # send HintText
            hudlib.hinttextsend(
                getUsers(parameter), es.getargs(3), time if time.isdigit() else 6)

        # return 0/1 if hud exists
        elif command == "exists":
            es.set(parameter, 1 if hintTextDict.has_key(es.getargv(3)) else 0)

        # delete a registered hud
        elif command == "delete":
            if hintTextDict.has_key(parameter):
                hintTextDict[parameter].kill()
                del hintTextDict[parameter]

        # create a hud for handling
        elif command == "create":
            if hintTextDict.has_key(parameter):
                hintTextDict[parameter].kill()
                del hintTextDict[parameter]
            hudbox = hintTextDict[parameter] = hudlib.hinttextbox()
            hudbox.shellname = parameter

        # if hud exists
        elif hintTextDict.has_key(parameter):
            # execute registered command (see under)
            regCommand(hintTextDict[parameter], command, parameter)

        # get if player is viewing a HintText box
        elif command == "view":
            es.set(parameter, 1 if hudlib.hinttextview(es.getargv(3)) else 0)

        # else: print debug msg
        else:
            es.dbgmsg(0, "Syntax: hinttext <command> [parameter(s)]")
            es.dbgmsg(0, 'HudLib: command "%s" does not exists' % command)



def keyHintTextCmd():
    count = es.getargc()
    if count < 3:
        es.dbgmsg(0, "\nHudLib: keyhinttext commands :\n\n  - " + "\n  - ".join(subcommands) + "\n")
    else:
        # get command and first parameter
        command = es.getargv(1)
        parameter = es.getargv(2)

        # for quick send
        if command == "quicksend":
            time = es.getargv(4)
            # send KeyHintText
            hudlib.keyhinttextsend(
                getUsers(parameter), es.getargs(3), time if time.isdigit() else 6)

        # return 0/1 if hud exists
        elif command == "exists":
            es.set(parameter, 1 if keyHintTextDict.has_key(es.getargv(3)) else 0)

        # delete a registered hud
        elif command == "delete":
            if keyHintTextDict.has_key(parameter):
                keyHintTextDict[parameter].kill()
                del keyHintTextDict[parameter]

        # create a hud for handling
        elif command == "create":
            if keyHintTextDict.has_key(parameter):
                keyHintTextDict[parameter].kill()
                del keyHintTextDict[parameter]
            hudbox = keyHintTextDict[parameter] = hudlib.keyhinttextbox()
            hudbox.shellname = parameter

        # if hudbox exists
        elif keyHintTextDict.has_key(parameter):
            # execute registered command (see under)
            regCommand(keyHintTextDict[parameter], command, parameter)

        # get if player is viewing a KeyHintText box
        elif command == "view":
            es.set(parameter, 1 if hudlib.keyhinttextview(es.getargv(3)) else 0)

        # else: print debug msg
        else:
            es.dbgmsg(0, "Syntax: keyhinttext <command> [parameter(s)]")
            es.dbgmsg(0, 'HudLib: command "%s" does not exists' % command)



def regCommand(hudbox, command, boxname):
    # send hud Key/HintText to player(s)
    if command == "send":
        hudbox.send(getUsers(es.getargv(3)))
    # hide hud Key/HintText to player(s)
    elif command == "hide":
        hudbox.hide(getUsers(es.getargv(3)))
    # update immediately hud Key/HintText of players
    elif command == "update":
        hudbox.update()

    # set the time the box will be displayed. 0 = until another takes its place
    elif command == "timeout":
        hudbox.timeout = int(es.getargv(3))
    # set a block to call when Key/HintText times out
    elif command == "boxtimeout":
        hudbox.boxblock = es.getargv(3)
        hudbox.boxtimeout = boxTimeOut
    # set a block to call when Key/HintText box of player is hidden
    elif command == "usertimeout":
        hudbox.userblock = es.getargv(3)
        hudbox.usertimeout = userBoxHide

    # load a language file, works with langlib module
    elif command == "language":
        directory = es.getargv(3)
        if directory.isdigit(): hudbox.language()
        else: hudbox.language(directory)
    # set default language to use when player language does not exists (langlib.getDefaultLang by default)
    elif command == "defaultlang":
        lang = es.getargv(3)
        hudbox.defaultlang = None if not lang or lang.isdigit() else lang

    # "index" parameter for insline/modline start at 0
    # add a line (text) in hud
    elif command == "addline":
        if hudbox._language is not False:
            section = es.getargv(3)
            keywords = getTokens(es.getargv(4), section)
            update = es.getargv(4) if keywords is None else es.getargv(5)
            hudbox.addline(section, keywords if keywords else {}, digitArg(update))
        else: hudbox.addline(es.getargv(3), {}, digitArg(es.getargv(4)))
    # insert a line (text) in hud with index
    elif command == "insline":
        if hudbox._language is not False:
            section = es.getargv(4)
            keywords = getTokens(es.getargv(5), section)
            update = es.getargv(5) if keywords is None else es.getargv(6)
            hudbox.insline(int(es.getargv(3)), section, keywords if keywords else {}, digitArg(update))
        else: hudbox.insline(int(es.getargv(3)), es.getargv(4), {}, digitArg(es.getargv(5)))
    # modifie a line (text) in hud with index
    elif command == "modline":
        if hudbox._language is not False:
            section = es.getargv(4)
            keywords = getTokens(es.getargv(5), section)
            update = es.getargv(5) if keywords is None else es.getargv(6)
            hudbox.modline(int(es.getargv(3)), section, keywords if keywords else {}, digitArg(update))
        else: hudbox.modline(int(es.getargv(3)), es.getargv(4), {}, digitArg(es.getargv(5)))
    # delete a line in hud with index.
    elif command == "delline":
        hudbox.delline(int(es.getargv(3)), digitArg(es.getargv(4)))

    # get if player is viewing this Key/HintText box (1/0)
    elif command == "view":
        es.set(es.getargv(3), 1 if hudbox.view(es.getargv(4)) else 0)
    # get a keygroup of userid of players that viewing this Key/HintText box
    elif command == "viewers":
        keygroupname = es.getargv(3)
        if es.exists("keygroup", keygroupname): es.keygroupdelete(keygroupname)
        es.keygroupcreate(keygroupname)
        for userid in hudbox.viewers(digitArg(es.getargv(4))): es.keycreate(keygroupname, userid)
    # store in the cvar the index (position) of a "line". start at 0 (list)
    elif command == "getindex":
        es.set(es.getargv(3), hudbox.lines.index(es.getargv(4)))
    # store in the cvar the text of a line. an index must be given (start at 0)
    elif command == "getstring":
        es.set(es.getargv(3), hudbox.lines[int(es.getargv(4))])
    # store in the cvar the number of line
    elif command == "getcount":
        es.set(es.getargv(3), len(hudbox.lines))

    # else: print debug msg
    else: es.dbgmsg(0, 'HudLib: command "%s" does not exists' % command)



def getTokens(keygroup, section = None):
    # open keygroup if it exists
    keygroup = keyvalues.getKeyGroup(keygroup)
    if section in keygroup:
        opts = {}
        key = keygroup[section]
        for token in key: opts[token.getName()] = key[token]
        # return options
        return opts
    return None

def boxTimeOut(hudbox):
    # set cvars
    cvHudname.set(hudbox.shellname)
    cvHudtype.set(hudbox.type)
    # execute a block when a box times out
    es.doblock(hudbox.boxblock)

def userBoxHide(userid, reason, hudbox):
    # set cvars
    cvUserid.set(userid)
    cvReason.set(reason)
    cvHudname.set(hudbox.shellname)
    cvHudtype.set(hudbox.type)
    # execute a block when a box is hidden from player
    es.doblock(hudbox.userblock)
